<?php
/**
 * Plugin Name: WooCommerce Adumo Online Gateway
 * Description: Receive payments through the secure Adumo Online payment gateway
 * Author: Adumo Online
 * Author URI: https://adumoonline.com/
 * Version: 2.0.10
 * Requires at least: 5.1
 * Tested up to: 5.9
 * WC tested up to: 6.4.3
 * WC requires at least: 4.5
 * Requires PHP: 7.0
 */

if (!defined('ABSPATH')) {
    exit;
}

include(dirname(__FILE__) . '/php-jwt/jwt.php');

function init_adumoonline_gateway()
{
    class WC_Gateway_AdumoOnline extends WC_Payment_Gateway
    {
        public function __construct()
        {
            $this->id = 'adumoonline';
            $this->icon = WP_PLUGIN_URL . '/woocommerce-adumoonline-gateway/img/Adumo_Online_logo.png';
            $this->has_fields = false;
            $this->method_title = __('Adumo Online Virtual', 'woocommerce-gateway-adumoonline');
            $this->method_description = __('Accept payments through Adumo Online\'s secure payment gateway', 'woocommerce-gateway-adumoonline');

            $this->init_form_fields();
            $this->init_settings();

            $this->title = $this->get_option('title');

            add_action('woocommerce_update_options_payment_gateways_' . $this->id, array($this, 'process_admin_options'));
            add_action('woocommerce_receipt_' . $this->id, array($this, 'receipt_page'));
        }

        public function init_form_fields()
        {
            $this->form_fields = array(
                'enabled' => array(
                    'title' => __('Enable/Disable', 'woocommerce-gateway-adumoonline'),
                    'type' => 'checkbox',
                    'label' => __('Enable Adumo Online Gateway', 'woocommerce-gateway-adumoonline'),
                    'default' => 'yes'
                ),
                'webhook' => array(
                    'title' => __('Activate Webhooks', 'woocommerce-gateway-adumoonline'),
                    'type' => 'checkbox',
                    'label' => __('Activate webhooks for this module. This will need to be configured in the Adumo Online Merchant Portal by adding this webhook endpoint - ' . get_site_url() . '/wp-json/adumoonline_webhook/v2/process', 'woocommerce-gateway-adumoonline'),
                    'default' => 'no'
                ),
                'jwt_secret' => array(
                    'title' => __('JWT Secret Key', 'woocommerce-gateway-adumoonline'),
                    'type' => 'text',
                    'description' => __('Your secret key to securly unpack the JWT token, this is configured in the Adumo Online Merchant Portal', 'woocommerce-gateway-adumoonline'),
                    'default' => 'yglTxLCSMm7PEsfaMszAKf2LSRvM2qVW'
                ),
                'title' => array(
                    'title' => __('Title', 'woocommerce'),
                    'type' => 'text',
                    'description' => __('This controls the title which the user sees during checkout.', 'woocommerce'),
                    'default' => __('Adumo Online Credit Card Payment', 'woocommerce'),
                    'desc_tip' => true,
                ),
                'gateway_endpoint' => array(
                    'title' => __('Gateway URL', 'woocommerce-gateway-adumoonline'),
                    'type' => 'text',
                    'description' => 'You can get the URL from the Adumo Online developer documentation https://developers.adumoonline.com/. eg: https://apiv3.adumoonline.com/product/payment/v1/initialisevirtual',
                    'default' => 'https://staging-apiv3.adumoonline.com/product/payment/v1/initialisevirtual'
                ),
                'merchant_id' => array(
                    'title' => __('Merchant ID', 'woocommerce-gateway-adumoonline'),
                    'type' => 'text',
                    'description' => 'You can get your merchant id by logging into your merchant portal at https://portal.adumoonline.com/login. eg: 9ba5008c-08ee-4286-a349-54af91a621b0',
                    'default' => '9ba5008c-08ee-4286-a349-54af91a621b0'
                ),
                'application_uid' => array(
                    'title' => __('Application UID', 'woocommerce-gateway-adumoonline'),
                    'type' => 'text',
                    'description' => 'You can get your application id by logging into your merchant portal at https://portal.adumoonline.com/login. eg: 4196b0b8-db88-42e5-a06d-294a5e4ded87',
                    'default' => '4196b0b8-db88-42e5-a06d-294a5e4ded87'
                )
            );
        }

        public function process_payment($order_id)
        {
            $order = wc_get_order($order_id);

            return array(
                'result' => 'success',
                'redirect' => $order->get_checkout_payment_url(true),
            );
        }

        public function receipt_page($order)
        {
            echo $this->generate_payment_form($order);
        }

        public function generate_payment_form($order_id)
        {
            $order = wc_get_order($order_id);

            //Build the JWT request token
            $MerchantID = $this->get_option('merchant_id');
            $ApplicationID = $this->get_option('application_uid');
            $amount = $order->get_total();
            $merchantReference = $order_id;
            $SecretKey = $this->get_option('jwt_secret');
            $iss = 'Woocommerce Plugin';
            $authToken = '';

            $payloadArray = array();
            $payloadArray['iss'] = $iss;
            $payloadArray['cuid'] = $MerchantID;
            $payloadArray['auid'] = $ApplicationID;
            $payloadArray['amount'] = $amount;
            $payloadArray['mref'] = $merchantReference;
            $payloadArray['jti'] = base64_encode(random_bytes(32));
            $payloadArray['iat'] = time() - 60;
            $payloadArray['exp'] = time() + 600;

            $authToken = JWT::encode($payloadArray, $SecretKey);

            $formFields = array(
                'Mode' => 1,
                'MerchantID' => $MerchantID,
                'ApplicationID' => $ApplicationID,
                'MerchantReference' => $merchantReference,
                'Amount' => $amount,
                'Token' => $authToken,
                'txtCurrencyCode' => $order->get_currency(),
                'redirectMethod'=>'POST',
                'RedirectSuccessfulURL' => $this->get_return_url($order),
                'RedirectFailedURL' => $this->get_return_url($order),
                'Discount' => number_format($this->get_order_details($order, 'discount_total'), 2),
                'ShippingCost' => number_format($this->get_order_details($order, 'shipping_total'), 2),
                'Recipient' => $this->get_order_details($order, 'billing_first_name') . ' ' . $this->get_order_details($order, 'billing_last_name'),
                'Email' => $this->get_order_details($order, 'billing_email'),
                'ContactNumber' => $this->get_order_details($order, 'billing_phone'),
                // 'MobileNumber' => $this->get_order_details($order, 'billing_phone'),
                'ShippingAddress1' => $this->get_order_details($order, 'shipping_address_1'),
                'ShippingAddress2' => $this->get_order_details($order, 'shipping_address_2'),
                'ShippingAddress3' => $this->get_order_details($order, 'shipping_city'),
                'ShippingAddress4' => $this->get_order_details($order, 'shipping_postcode'),
                'ShippingAddress5' => $this->get_order_details($order, 'shipping_country'),
            );

            $counter = 1;
            foreach ($order->get_items() as $product) {
                /** @var WC_Order_Item_Product $product */

                $formFields["Qty$counter"] = $product->get_quantity();
                $formFields["ItemRef$counter"] = $product->get_id();
                $formFields["ItemDescr$counter"] = $product->get_name();
                $formFields["ItemAmount$counter"] = number_format($product->get_total(), 2);

                $counter++;
            }

            $form = '<form action="' . esc_url($this->get_option('gateway_endpoint')) . '" method="post" id="adumoonline_payment_form">';

            foreach ($formFields as $key => $value) {
                $form .= '<input type="hidden" name="' . esc_attr($key) . '" value="' . esc_attr($value) . '" />';
            }

            $form .= '
				<input type="submit" class="button-alt" id="submit_adumoonline_payment_form" value="' . __('Pay with Credit Card', 'woocommerce-gateway-adumoonline') . '" /> <a class="button cancel" href="' . $order->get_cancel_order_url() . '">' . __('Cancel order &amp; restore cart', 'woocommerce-gateway-adumoonline') . '</a>
				<script type="text/javascript">
					jQuery(function(){
						jQuery("body").block(
							{
								message: "<img src=\"' . $this->icon . '\" /><br />' . __('Thank you for your order. You will now be redirected to Adumo Online\'s secure payment page to make payment.', 'woocommerce-gateway-adumoonline') . '",
								overlayCSS:
								{
									background: "#fff",
									opacity: 0.6
								},
								css: {
									padding:        20,
									textAlign:      "center",
									color:          "#555",
									border:         "3px solid #aaa",
									backgroundColor:"#fff",
									cursor:         "wait"
								}
							});
						jQuery( "#submit_adumoonline_payment_form" ).click();
					});
				</script>
			</form>';

            return $form;
        }

        public static function check_status_response()
        {
            $logger = wc_get_logger();

            //Check if JWT token is in the response and that a secret key is set
            if (isset($_POST['_RESPONSE_TOKEN']) && !empty($_POST['_RESPONSE_TOKEN'])) {
                $logger->debug("Reponse received: " . print_r($_POST, 1));
                $jwtToken = $_POST['_RESPONSE_TOKEN'];

                if (isset($_GET['key'])) {
                    $key = $_GET['key'];
                } else {
                    $key = null;
                }
                process_the_order($key, $jwtToken, 'browser', $_POST);

            } else {
                //No token in the response
                if (isset($_POST['_MERCHANTREFERENCE'])) {
                    $order_id = (int) (isset($_POST['_MERCHANTREFERENCE']) ? $_POST['_MERCHANTREFERENCE'] : 0);
                    $order = wc_get_order($order_id);
                    if ($order->has_status(array('processing', 'complete'))) {
                        $order->add_order_note(__(sprintf('Order Already processed'), 'woocommerce-gateway-adumoonline'));
                        return;
                    }
                    if (isset($_POST['_RESULT']) && $_POST['_RESULT'] < 0) {

                        $logger->warning($_POST['_ERROR_DETAIL']);
                        $message = $_POST['_ERROR_DETAIL'];
                        $order->update_status('failed', __(sprintf('Adumo Online payment failed: %s', $message), 'woocommerce-gateway-adumoonline'));
                    } else {
                        $logger->error('JWT Token not found in the response from AdumoOnline!');
                        $message = 'JWT Token not found in the response from AdumoOnline!';
                        $order->update_status('failed', __(sprintf('Adumo Online payment failed: %s', $message), 'woocommerce-gateway-adumoonline'));
                    }

                } else {
                    
                    // $logger->error("******* Invalid Reponse: " . print_r($_POST, 1));
                }


            }

        }

        private function get_order_details($order, $prop)
        {
            switch ($prop) {
                case 'order_total':
                    $getter = array($order, 'get_total');
                    break;
                default:
                    $getter = array($order, 'get_' . $prop);
                    break;
            }

            return is_callable($getter) ? call_user_func($getter) : $order->{$prop};
        }
    }

    add_filter('woocommerce_payment_gateways', 'add_adumoonline_gateway_class');
    add_filter('plugin_action_links_' . plugin_basename(__FILE__), 'woocommerce_adumoonline_plugin_links');
}

function process_the_order($key, $jwtToken, $type, $response)
{
    $logger = wc_get_logger();
    //Get admin settings and validate the JWT token
    $gateway_settings = WC()->payment_gateways->payment_gateways()['adumoonline'];
    $jwtSecret = $gateway_settings->settings['jwt_secret'];

    $data = JWT::decode($jwtToken, $jwtSecret, array('HS256'));
    $logger->debug("JWT Response-----------------------------------" . print_r($data, 1));
    $mref = $data->mref;
    $result = $data->result;
    $transactionIndex = $data->transactionIndex;
    $amount = $data->amount;
    $valid = $data->verify;

    $order_id = (int) (isset($mref) ? $mref : 0);

    if (0 === $order_id) {
        return;
    }

    $order = wc_get_order($order_id);

    if ($order->has_status(array('processing', 'complete'))) {
        return;
    }

    $result = isset($result) ? (int) $result : false;

    if (false === $result) {
        return;
    }

    if ($key !== $order->get_order_key() && $type == 'browser') {
        return wp_redirect($order->get_cancel_order_url());
    }

    if ($result >= 0) {
        $transactionIndex = isset($transactionIndex) ? $transactionIndex : 'N/A';
        $order->add_order_note(__(sprintf('Adumo Online payment completed. Transaction Id: %s', $transactionIndex), 'woocommerce-gateway-adumoonline'));
        $order->payment_complete($transactionIndex);

        //Validate the token
        if ($valid == 1) {
            $logger->info("JWT Token Verified");
            $order->add_order_note(__(sprintf('JWT Token Verified'), 'woocommerce-gateway-adumoonline'));
        } else {
            $logger->warning('JWT Token does not have valid signature!');
            $order->add_order_note(__(sprintf('JWT Token does not have valid signature! Reason: %s', $data->reason), 'woocommerce-gateway-adumoonline'));
            $order->update_status('on-hold');
            return;
        }

        //Validate the amount
        if ($order->get_total() == $amount) {
            $logger->info("Amount validation passed");
            $order->add_order_note(__(sprintf('Amount validation passed'), 'woocommerce-gateway-adumoonline'));
        } else {
            $logger->warning('Amount validation failed - order amount and gateway amount are different!');
            $order->add_order_note(__(sprintf('Amount validation failed - order amount and gateway amount are different!'), 'woocommerce-gateway-adumoonline'));
            $order->update_status('on-hold');
            return;
        }

    } else {
        $logger->debug("Error message: " . print_r($response, 1));
        $message = '';
        if (!empty($response['_BANK_ERROR_MESSAGE'])) {
            $message = $response['_BANK_ERROR_MESSAGE'];
            $logger->debug("_BANK_ERROR_MESSAGE is set. Value: '" . $message . "'");
        } elseif (!empty($response['_ERROR_DETAIL'])) {
            $message = $response['_ERROR_DETAIL'];
            $logger->debug("_ERROR_DETAIL is set. Value: '" . $message . "'");
        } elseif (!empty($response['_ERROR_MESSAGE'])) {
            $message = $response['_ERROR_MESSAGE'];
            $logger->debug("_ERROR_MESSAGE is set. Value: '" . $message . "'");
        } elseif (!empty($response['message'])) {
            $message = $response['message'];
            $logger->debug("_ERROR_MESSAGE is set. Value: '" . $message . "'");
        } else {
            $message = 'Unknown Error';
            $logger->debug("No known error fields set. Using default message.");
        }
        
        $logger->debug("Final message value: '" . $message . "'");

        $order->add_order_note(__(sprintf('Declined: %s', $message), 'woocommerce-gateway-adumoonline'));
        $order->update_status('failed', __(sprintf('Adumo Online payment failed: %s', $message), 'woocommerce-gateway-adumoonline'));
    }
}


function add_adumoonline_gateway_class($methods)
{
    $methods[] = 'WC_Gateway_AdumoOnline';

    return $methods;
}

add_action('plugins_loaded', 'init_adumoonline_gateway', 0);
add_action('init', array('WC_Gateway_AdumoOnline', 'check_status_response'), 255);

/**
 * Checks if the WooCommerce plugin is activated
 *
 * If the WooCommerce plugin is not active, then don't allow the
 * activation of this plugin.
 *
 * @since 1.0.0
 */
function adumoonline_activate()
{
    if (!function_exists('is_plugin_active_for_network')) {
        include_once(ABSPATH . '/wp-admin/includes/plugin.php');
    }
    if (current_user_can('activate_plugins') && (!class_exists('WooCommerce'))) {
        // Deactivate the plugin.
        deactivate_plugins(plugin_basename(__FILE__));
        // Throw an error in the WordPress admin console.
        $error_message = '<p style="font-family:-apple-system,BlinkMacSystemFont,\'Segoe UI\',Roboto,Oxygen-Sans,Ubuntu,Cantarell,\'Helvetica Neue\',sans-serif;font-size: 13px;line-height: 1.5;color:#444;">' . esc_html__('This plugin requires ', 'adumoonline') . '<a href="' . esc_url('https://woocommerce.com/') . '">WooCommerce</a>' . esc_html__(' to be active.', 'adumoonline') . '</p>';
        die($error_message);
    }
}
register_activation_hook(__FILE__, 'adumoonline_activate');

//Setup webhook endpoint
function adumoonline_webhook_v2(WP_REST_Request $request)
{
    try{
        $logger = wc_get_logger();
        $logger->debug('Webhook Received: ' . print_r($request->get_params(), true));
        $data = $request->get_params();
        $order = wc_get_order($data['merchantReference']);
        
        if (!$order) {
            $logger->error('Order not found for merchantReference: ' . $data['merchantReference']);
            return new WP_REST_Response(
                array(
                    'status' => 'error',
                    'message' => 'Order not found'
                ),
                404
            );
        }

        $jwtToken = $data['token'];

        process_the_order('', $jwtToken, 'webhook', $data);

        // Get the updated order status after processing
        $order_status = $order->get_status();

        $logger->debug('Webhook processing complete. Order status: ' . $order_status);

        return new WP_REST_Response(
            array(
                'status' => 'success',
                'message' => 'Webhook processed successfully',
                'order_status' => $order_status
            ),
            200
        );
    } catch (Exception $e) {
        $logger->error('Error processing webhook: ' . $e->getMessage());
        return new WP_Error(
            'error',
            $e->getMessage(),
            array('status' => 500)
        );
    }
}



//Register the webhook endpoint v1
function register_adumoonline_webhooks_routes_v2()
{
    register_rest_route('adumoonline_webhook/v2', '/process', array(
        'methods' => 'POST', //WP_REST_Server::READABLE,
        'callback' => 'adumoonline_webhook_v2',
        'permission_callback' => '__return_true'
    ));
}
add_action('rest_api_init', 'register_adumoonline_webhooks_routes_v2');

/**
 * Declares compatibility with Woocommerce features.
 */
function woocommerce_adumoonline_declare_feature_compatibility()
{
    if (class_exists(\Automattic\WooCommerce\Utilities\FeaturesUtil::class)) {
        \Automattic\WooCommerce\Utilities\FeaturesUtil::declare_compatibility(
            'custom_order_tables',
            __FILE__
        );

        \Automattic\WooCommerce\Utilities\FeaturesUtil::declare_compatibility(
            'product_block_editor',
            __FILE__
        );
    }
}
add_action('before_woocommerce_init', 'woocommerce_adumoonline_declare_feature_compatibility');


function woocommerce_adumoonline_plugin_links($links)
{
    $settings_url = add_query_arg(
        array(
            'page' => 'wc-settings',
            'tab' => 'checkout',
            'section' => 'wc_gateway_adumoonline',
        ),
        admin_url('admin.php')
    );

    $plugin_links = array(
        '<a href="' . esc_url($settings_url) . '">' . esc_html__('Settings', 'woocommerce-gateway-adumoonline') . '</a>',
        '<a href="https://developers.adumoonline.com/virtual.php#htmlsample"  target="_blank" rel="noopener noreferrer">' . esc_html__('Docs', 'woocommerce-gateway-adumoonline') . '</a>',
    );

    return array_merge($plugin_links, $links);
}

function woocommerce_adumoonline_missing_wc_notice()
{
    if (class_exists('WooCommerce')) {
        // Display nothing if WooCommerce is installed and activated.
        return;
    }

    echo '<div class="error"><p><strong>';
    printf(
        /* translators: %s WooCommerce download URL link. */
        esc_html__('Adumo Online plugin requires WooCommerce to be installed and active. You can download %s here.', 'woocommerce-gateway-adumoonline'),
        '<a href="https://woocommerce.com/" target="_blank">WooCommerce</a>'
    );
    echo '</strong></p></div>';
}
add_action('admin_notices', 'woocommerce_adumoonline_missing_wc_notice');